package sun.security.tools.policytool;

import javax.swing.UIManager;
import java.awt.event.KeyEvent;
import javax.swing.SwingUtilities;
import java.lang.reflect.Constructor;
import java.security.Permission;
import javax.security.auth.x500.X500Principal;
import java.security.cert.Certificate;
import java.security.PublicKey;
import java.util.LinkedList;
import javax.security.auth.login.LoginException;
import sun.security.util.PolicyUtil;
import java.net.URL;
import java.io.File;
import java.io.FileWriter;
import sun.security.util.PropertyExpander;
import java.security.NoSuchProviderException;
import java.security.UnrecoverableKeyException;
import java.security.NoSuchAlgorithmException;
import java.io.IOException;
import java.net.MalformedURLException;
import java.security.cert.CertificateException;
import java.security.KeyStoreException;
import java.io.FileNotFoundException;
import java.util.ListIterator;
import java.util.Enumeration;
import java.lang.reflect.InvocationTargetException;
import java.text.MessageFormat;
import java.io.FileReader;
import java.security.KeyStore;
import sun.security.provider.PolicyParser;
import java.util.Vector;
import java.text.Collator;
import java.util.ResourceBundle;

public class PolicyTool
{
    static final ResourceBundle rb;
    static final Collator collator;
    Vector<String> warnings;
    boolean newWarning;
    boolean modified;
    private static final boolean testing = false;
    private static final Class<?>[] TWOPARAMS;
    private static final Class<?>[] ONEPARAMS;
    private static final Class<?>[] NOPARAMS;
    private static String policyFileName;
    private Vector<PolicyEntry> policyEntries;
    private PolicyParser parser;
    private KeyStore keyStore;
    private String keyStoreName;
    private String keyStoreType;
    private String keyStoreProvider;
    private String keyStorePwdURL;
    private static final String P11KEYSTORE = "PKCS11";
    private static final String NONE = "NONE";
    
    private PolicyTool() {
        this.newWarning = false;
        this.modified = false;
        this.policyEntries = null;
        this.parser = null;
        this.keyStore = null;
        this.keyStoreName = " ";
        this.keyStoreType = " ";
        this.keyStoreProvider = " ";
        this.keyStorePwdURL = " ";
        this.policyEntries = new Vector();
        this.parser = new PolicyParser();
        this.warnings = new Vector();
    }
    
    String getPolicyFileName() {
        return PolicyTool.policyFileName;
    }
    
    void setPolicyFileName(String policyFileName) {
        PolicyTool.policyFileName = policyFileName;
    }
    
    void clearKeyStoreInfo() {
        this.keyStoreName = null;
        this.keyStoreType = null;
        this.keyStoreProvider = null;
        this.keyStorePwdURL = null;
        this.keyStore = null;
    }
    
    String getKeyStoreName() {
        return this.keyStoreName;
    }
    
    String getKeyStoreType() {
        return this.keyStoreType;
    }
    
    String getKeyStoreProvider() {
        return this.keyStoreProvider;
    }
    
    String getKeyStorePwdURL() {
        return this.keyStorePwdURL;
    }
    
    void openPolicy(String s) throws FileNotFoundException, PolicyParser.ParsingException, KeyStoreException, CertificateException, InstantiationException, MalformedURLException, IOException, NoSuchAlgorithmException, IllegalAccessException, NoSuchMethodException, UnrecoverableKeyException, NoSuchProviderException, ClassNotFoundException, PropertyExpander.ExpandException, InvocationTargetException {
        this.newWarning = false;
        this.policyEntries = new Vector();
        this.parser = new PolicyParser();
        this.warnings = new Vector();
        this.setPolicyFileName(null);
        this.clearKeyStoreInfo();
        if (s == null) {
            this.modified = false;
            return;
        }
        this.setPolicyFileName(s);
        this.parser.read(new FileReader(s));
        this.openKeyStore(this.parser.getKeyStoreUrl(), this.parser.getKeyStoreType(), this.parser.getKeyStoreProvider(), this.parser.getStorePassURL());
        Enumeration<PolicyParser.GrantEntry> grantElements = this.parser.grantElements();
        while (grantElements.hasMoreElements()) {
            PolicyParser.GrantEntry grantEntry = (PolicyParser.GrantEntry)grantElements.nextElement();
            if (grantEntry.signedBy != null) {
                String[] signers = this.parseSigners(grantEntry.signedBy);
                for (int i = 0; i < signers.length; ++i) {
                    if (this.getPublicKeyAlias(signers[i]) == null) {
                        this.newWarning = true;
                        this.warnings.addElement(new MessageFormat(getMessage("Warning.A.public.key.for.alias.signers.i.does.not.exist.Make.sure.a.KeyStore.is.properly.configured.")).format(new Object[] { signers[i] }));
                    }
                }
            }
            ListIterator<PolicyParser.PrincipalEntry> listIterator = grantEntry.principals.listIterator(0);
            while (listIterator.hasNext()) {
                PolicyParser.PrincipalEntry principalEntry = (PolicyParser.PrincipalEntry)listIterator.next();
                try {
                    this.verifyPrincipal(principalEntry.getPrincipalClass(), principalEntry.getPrincipalName());
                }
                catch (ClassNotFoundException ex) {
                    this.newWarning = true;
                    this.warnings.addElement(new MessageFormat(getMessage("Warning.Class.not.found.class")).format(new Object[] { principalEntry.getPrincipalClass() }));
                }
            }
            Enumeration<PolicyParser.PermissionEntry> permissionElements = grantEntry.permissionElements();
            while (permissionElements.hasMoreElements()) {
                PolicyParser.PermissionEntry permissionEntry = (PolicyParser.PermissionEntry)permissionElements.nextElement();
                try {
                    this.verifyPermission(permissionEntry.permission, permissionEntry.name, permissionEntry.action);
                }
                catch (ClassNotFoundException ex2) {
                    this.newWarning = true;
                    this.warnings.addElement(new MessageFormat(getMessage("Warning.Class.not.found.class")).format(new Object[] { permissionEntry.permission }));
                }
                catch (InvocationTargetException ex3) {
                    this.newWarning = true;
                    this.warnings.addElement(new MessageFormat(getMessage("Warning.Invalid.argument.s.for.constructor.arg")).format(new Object[] { permissionEntry.permission }));
                }
                if (permissionEntry.signedBy != null) {
                    String[] signers2 = this.parseSigners(permissionEntry.signedBy);
                    for (int j = 0; j < signers2.length; ++j) {
                        if (this.getPublicKeyAlias(signers2[j]) == null) {
                            this.newWarning = true;
                            this.warnings.addElement(new MessageFormat(getMessage("Warning.A.public.key.for.alias.signers.i.does.not.exist.Make.sure.a.KeyStore.is.properly.configured.")).format(new Object[] { signers2[j] }));
                        }
                    }
                }
                else {
                    continue;
                }
            }
            this.policyEntries.addElement(new PolicyEntry(this, grantEntry));
        }
        this.modified = false;
        return;
    }
    
    void savePolicy(String fileName) throws FileNotFoundException, IOException {
        this.parser.setKeyStoreUrl(this.keyStoreName);
        this.parser.setKeyStoreType(this.keyStoreType);
        this.parser.setKeyStoreProvider(this.keyStoreProvider);
        this.parser.setStorePassURL(this.keyStorePwdURL);
        this.parser.write(new FileWriter(fileName));
        this.modified = false;
    }
    
    void openKeyStore(String replace, String defaultType, String s, String replace2) throws KeyStoreException, NoSuchAlgorithmException, UnrecoverableKeyException, IOException, CertificateException, NoSuchProviderException, PropertyExpander.ExpandException {
        if (replace == null && defaultType == null && s == null && replace2 == null) {
            this.keyStoreName = null;
            this.keyStoreType = null;
            this.keyStoreProvider = null;
            this.keyStorePwdURL = null;
            return;
        }
        URL policyUrl = null;
        if (PolicyTool.policyFileName != null) {
            policyUrl = new URL("file:" + new File(PolicyTool.policyFileName).getCanonicalPath());
        }
        if (replace != null && replace.length() > 0) {
            replace = PropertyExpander.expand(replace).replace(File.separatorChar, '/');
        }
        if (defaultType == null || defaultType.length() == 0) {
            defaultType = KeyStore.getDefaultType();
        }
        if (replace2 != null && replace2.length() > 0) {
            replace2 = PropertyExpander.expand(replace2).replace(File.separatorChar, '/');
        }
        try {
            this.keyStore = PolicyUtil.getKeyStore(policyUrl, replace, defaultType, s, replace2, null);
        }
        catch (IOException ex) {
            String message = "no password provided, and no callback handler available for retrieving password";
            Throwable cause = ex.getCause();
            if (cause != null && cause instanceof LoginException && message.equals(cause.getMessage())) {
                throw new IOException(message);
            }
            throw ex;
        }
        this.keyStoreName = replace;
        this.keyStoreType = defaultType;
        this.keyStoreProvider = s;
        this.keyStorePwdURL = replace2;
        return;
    }
    
    boolean addEntry(PolicyEntry policyEntry, int n) {
        if (n < 0) {
            this.policyEntries.addElement(policyEntry);
            this.parser.add(policyEntry.getGrantEntry());
        }
        else {
            this.parser.replace(((PolicyEntry)this.policyEntries.elementAt(n)).getGrantEntry(), policyEntry.getGrantEntry());
            this.policyEntries.setElementAt(policyEntry, n);
        }
        return true;
    }
    
    boolean addPrinEntry(PolicyEntry policyEntry, PolicyParser.PrincipalEntry element, int index) {
        PolicyParser.GrantEntry grantEntry = policyEntry.getGrantEntry();
        if (grantEntry.contains(element)) {
            return false;
        }
        LinkedList principals = grantEntry.principals;
        if (index != -1) {
            principals.set(index, element);
        }
        else {
            principals.add(element);
        }
        return this.modified = true;
    }
    
    boolean addPermEntry(PolicyEntry policyEntry, PolicyParser.PermissionEntry obj, int index) {
        PolicyParser.GrantEntry grantEntry = policyEntry.getGrantEntry();
        if (grantEntry.contains(obj)) {
            return false;
        }
        Vector<PolicyParser.PermissionEntry> permissionEntries = grantEntry.permissionEntries;
        if (index != -1) {
            permissionEntries.setElementAt(obj, index);
        }
        else {
            permissionEntries.addElement(obj);
        }
        return this.modified = true;
    }
    
    boolean removePermEntry(PolicyEntry policyEntry, PolicyParser.PermissionEntry pe) {
        return this.modified = policyEntry.getGrantEntry().remove(pe);
    }
    
    boolean removeEntry(PolicyEntry obj) {
        this.parser.remove(obj.getGrantEntry());
        this.modified = true;
        return this.policyEntries.removeElement(obj);
    }
    
    PolicyEntry[] getEntry() {
        if (this.policyEntries.size() > 0) {
            PolicyEntry[] array = new PolicyEntry[this.policyEntries.size()];
            for (int i = 0; i < this.policyEntries.size(); ++i) {
                array[i] = (PolicyEntry)this.policyEntries.elementAt(i);
            }
            return array;
        }
        return null;
    }
    
    PublicKey getPublicKeyAlias(String alias) throws KeyStoreException {
        if (this.keyStore == null) {
            return null;
        }
        Certificate certificate = this.keyStore.getCertificate(alias);
        if (certificate == null) {
            return null;
        }
        return certificate.getPublicKey();
    }
    
    String[] getPublicKeyAlias() throws KeyStoreException {
        int n = 0;
        String[] array = null;
        if (this.keyStore == null) {
            return null;
        }
        for (Enumeration aliases = this.keyStore.aliases(); aliases.hasMoreElements(); ++n) {
            aliases.nextElement();
        }
        if (n > 0) {
            array = new String[n];
            int n2 = 0;
            Enumeration<String> aliases2 = this.keyStore.aliases();
            while (aliases2.hasMoreElements()) {
                array[n2] = new String((String)aliases2.nextElement());
                ++n2;
            }
            return array;
        }
        return array;
    }
    
    String[] parseSigners(String s) {
        int n = 1;
        int fromIndex = 0;
        int i = 0;
        int n2 = 0;
        while (i >= 0) {
            i = s.indexOf(44, fromIndex);
            if (i < 0) {
                continue;
            }
            ++n;
            fromIndex = i + 1;
        }
        String[] array = new String[n];
        int j = 0;
        int beginIndex = 0;
        while (j >= 0) {
            if ((j = s.indexOf(44, beginIndex)) >= 0) {
                array[n2] = s.substring(beginIndex, j).trim();
                ++n2;
                beginIndex = j + 1;
            }
            else {
                array[n2] = s.substring(beginIndex).trim();
            }
        }
        return array;
    }
    
    void verifyPrincipal(String name, String name2) throws ClassNotFoundException, InstantiationException {
        if (name.equals("WILDCARD_PRINCIPAL_CLASS") || name.equals("PolicyParser.REPLACE_NAME")) {
            return;
        }
        Class<Object> forName = Class.forName("java.security.Principal");
        Class<Object> forName2 = Class.forName(name, true, Thread.currentThread().getContextClassLoader());
        if (!forName.isAssignableFrom(forName2)) {
            throw new InstantiationException(new MessageFormat(getMessage("Illegal.Principal.Type.type")).format(new Object[] { name }));
        }
        if ("javax.security.auth.x500.X500Principal".equals(forName2.getName())) {
            X500Principal x500Principal = new X500Principal(name2);
        }
        return;
    }
    
    void verifyPermission(String name, String e, String e2) throws ClassNotFoundException, InstantiationException, IllegalAccessException, NoSuchMethodException, InvocationTargetException {
        Class<Object> forName = Class.forName(name, true, Thread.currentThread().getContextClassLoader());
        Constructor<Object> constructor = null;
        Vector vector = new Vector(2);
        if (e != null) {
            vector.add(e);
        }
        if (e2 != null) {
            vector.add(e2);
        }
        switch (vector.size()) {
            case 0: {
                try {
                    constructor = forName.getConstructor(PolicyTool.NOPARAMS);
                    break;
                }
                catch (NoSuchMethodException ex) {
                    vector.add(null);
                }
            }
            case 1: {
                try {
                    constructor = forName.getConstructor(PolicyTool.ONEPARAMS);
                    break;
                }
                catch (NoSuchMethodException ex2) {
                    vector.add(null);
                }
            }
            case 2: {
                constructor = forName.getConstructor(PolicyTool.TWOPARAMS);
                break;
            }
        }
        Permission permission = (Permission)constructor.newInstance(vector.toArray());
    }
    
    static void parseArgs(String[] array) {
        for (int n = 0; n < array.length && array[n].startsWith("-"); ++n) {
            String s = array[n];
            if (PolicyTool.collator.compare(s, "-file") == 0) {
                if (++n == array.length) {
                    usage();
                }
                PolicyTool.policyFileName = array[n];
            }
            else {
                System.err.println(new MessageFormat(getMessage("Illegal.option.option")).format(new Object[] { s }));
                usage();
            }
        }
    }
    
    static void usage() {
        System.out.println(getMessage("Usage.policytool.options."));
        System.out.println();
        System.out.println(getMessage(".file.file.policy.file.location"));
        System.out.println();
        System.exit(1);
    }
    
    public static void main(final String[] array) {
        parseArgs(array);
        SwingUtilities.invokeLater(new Runnable() {
            @Override
            public void run() {
                new ToolWindow(new PolicyTool()).displayToolWindow(array);
            }
        });
    }
    
    static String splitToWords(String s) {
        return s.replaceAll("([A-Z])", " $1");
    }
    
    static String getMessage(String key) {
        return removeMnemonicAmpersand(PolicyTool.rb.getString(key));
    }
    
    static int getMnemonicInt(String key) {
        return findMnemonicInt(PolicyTool.rb.getString(key));
    }
    
    static int getDisplayedMnemonicIndex(String key) {
        return findMnemonicIndex(PolicyTool.rb.getString(key));
    }
    
    private static int findMnemonicInt(String s) {
        for (int i = 0; i < s.length() - 1; ++i) {
            if (s.charAt(i) == '&') {
                if (s.charAt(i + 1) != '&') {
                    return KeyEvent.getExtendedKeyCodeForChar((int)s.charAt(i + 1));
                }
                ++i;
            }
        }
        return 0;
    }
    
    private static int findMnemonicIndex(String s) {
        for (int i = 0; i < s.length() - 1; ++i) {
            if (s.charAt(i) == '&') {
                if (s.charAt(i + 1) != '&') {
                    return i;
                }
                ++i;
            }
        }
        return -1;
    }
    
    private static String removeMnemonicAmpersand(String s) {
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < s.length(); ++i) {
            int char1 = s.charAt(i);
            if (char1 != '&' || i == s.length() - 1 || s.charAt(i + 1) == '&') {
                sb.append((char)char1);
            }
        }
        return sb.toString();
    }
    
    PolicyTool(PolicyTool.1 runnable) {
        this();
    }
    
    static {
        rb = ResourceBundle.getBundle("sun.security.tools.policytool.Resources");
        (collator = Collator.getInstance()).setStrength(0);
        if (System.getProperty("apple.laf.useScreenMenuBar") == null) {
            System.setProperty("apple.laf.useScreenMenuBar", "true");
        }
        System.setProperty("apple.awt.application.name", getMessage("Policy.Tool"));
        if (System.getProperty("swing.defaultlaf") == null) {
            try {
                UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
            }
            catch (Exception ex) {}
        }
        TWOPARAMS = new Class[] { String.class, String.class };
        ONEPARAMS = new Class[] { String.class };
        NOPARAMS = new Class[0];
        PolicyTool.policyFileName = null;
    }
}
